import click
import subprocess
from .core.cardanopy_config import CardanoPyConfig
from .core.substitution import Substitution


@click.command("run")
@click.option('-d', '--dry-run', 'dry_run', is_flag=True, help="print the mutable commands")
@click.option('-b', '--block-producer', 'block_producer', is_flag=True, help="enable block producer mode")
@click.option('-s', '--sub', 'subs', multiple=True, type=str, default=tuple(), help="Substitutions for configs")
@click.argument('target_config_dir_or_file', type=str)
@click.pass_context
def run_cmd(ctx, dry_run, block_producer, subs, target_config_dir_or_file):
    """Run command"""

    try:
        target_config_dir = CardanoPyConfig.try_get_valid_config_dir(target_config_dir_or_file)
        target_config_file = CardanoPyConfig.try_get_valid_config_file(target_config_dir_or_file)

        cardanopy_config = CardanoPyConfig()
        cardanopy_config.load(target_config_file, subs)

        Substitution.generate(dry_run, target_config_dir, cardanopy_config, subs)

        cardano_node_cmd = ["cardano-node",
                            "run",
                            "--config", cardanopy_config.configPath,
                            "--topology", cardanopy_config.topologyPath,
                            "--database-path", cardanopy_config.databasePath,
                            "--host-addr", cardanopy_config.hostAddr,
                            "--port", f"{cardanopy_config.port}",
                            "--socket-path", cardanopy_config.socketPath]

        if block_producer:
            cardano_node_cmd = cardano_node_cmd + ["--shelley-kes-key", cardanopy_config.shelleyKesKey,
                                                   "--shelley-vrf-key", cardanopy_config.shelleyVrfKey,
                                                   "--shelley-operational-certificate", cardanopy_config.shelleyOperationalCertificate]

        if dry_run:
            print(" ".join(cardano_node_cmd))
        else:
            subprocess.run(cardano_node_cmd, cwd=target_config_dir)
    except Exception as ex:
        ctx.fail(f"run_cmd(dry_run={dry_run}, target_config_dir_or_file='{target_config_dir_or_file}') failed: {type(ex).__name__} {ex.args}")
        return 1




import unittest
from core.cardanopy_config import CardanoPyConfig
from core.cardanopy_common import CardanoPyCommon
from pathlib import Path
import tempfile
import shutil
import os
import yaml
import json
import jsonschema


class TestCardanoPyConfig(unittest.TestCase):

    def setUp(self):
        self.test_dir = Path(tempfile.mkdtemp())
        os.chdir(self.test_dir)
        # print(f"test_dir='{self.test_dir}'")

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    def test_basic_config_validation(self):
        config_file = Path(__file__).parent.joinpath("../../data/templates").absolute() / "testnet/basic/cardanopy.yaml"
        with open(config_file, "r") as file:
            config = yaml.full_load(file.read())

        api_version = config["apiVersion"]

        schema_file = Path(__file__).parent.joinpath("../../data/schemas").absolute() / f"{api_version}.json"
        with open(schema_file, "r") as file:
            schema = json.loads(file.read())

        jsonschema.validate(instance=config, schema=schema)

    def test_basic_config(self):
        file = Path(__file__).parent.joinpath("../../data/templates").absolute() / "testnet/basic/cardanopy.yaml"
        cardanopy_config = CardanoPyConfig()
        cardanopy_config.load(file)

        self.assertEqual(cardanopy_config.apiVersion, "cardanopy.node.config.v1")
        self.assertEqual(cardanopy_config.network, "testnet")
        self.assertEqual(cardanopy_config.configPath, "/home/ada/app/config.json")
        self.assertEqual(cardanopy_config.topologyPath, "/home/ada/app/topology.json")
        self.assertEqual(cardanopy_config.databasePath, "/home/ada/app/db")
        self.assertEqual(cardanopy_config.socketPath, "/home/ada/app/node.socket")
        self.assertEqual(cardanopy_config.hostAddr, "0.0.0.0")
        self.assertEqual(cardanopy_config.port, 3001)
        self.assertEqual(cardanopy_config.docker.name, "basic")
        self.assertEqual(cardanopy_config.docker.image, f"floydcraft/cardano-py-slim:{CardanoPyCommon.get_version()}")
        self.assertEqual(cardanopy_config.docker.rootVolume, "/home/ada/app")
        self.assertEqual(cardanopy_config.docker.mount, True)
        self.assertEqual(cardanopy_config.k8s.namespace, "cardano-testnet")

        file_save = self.test_dir / file.name
        cardanopy_config.save(file_save)

        cardanopy_config_v2 = CardanoPyConfig()
        cardanopy_config_v2.load(file_save)

        self.assertEqual(cardanopy_config_v2.apiVersion, cardanopy_config.apiVersion)
        self.assertEqual(cardanopy_config_v2.network, cardanopy_config.network)
        self.assertEqual(cardanopy_config_v2.configPath, cardanopy_config.configPath)
        self.assertEqual(cardanopy_config_v2.topologyPath, cardanopy_config.topologyPath)
        self.assertEqual(cardanopy_config_v2.databasePath, cardanopy_config.databasePath)
        self.assertEqual(cardanopy_config_v2.socketPath, cardanopy_config.socketPath)
        self.assertEqual(cardanopy_config_v2.hostAddr, cardanopy_config.hostAddr)
        self.assertEqual(cardanopy_config_v2.port, cardanopy_config.port)
        self.assertEqual(cardanopy_config_v2.docker.name, cardanopy_config.docker.name)
        self.assertEqual(cardanopy_config_v2.docker.image, cardanopy_config.docker.image)
        self.assertEqual(cardanopy_config_v2.docker.rootVolume, cardanopy_config.docker.rootVolume)
        self.assertEqual(cardanopy_config_v2.docker.mount, cardanopy_config.docker.mount)
        self.assertEqual(cardanopy_config_v2.k8s.namespace, cardanopy_config.k8s.namespace)

    def test_basic_config_subs(self):
        dir_path = Path(__file__).parent.joinpath("../../data/templates").absolute()
        for file in dir_path.glob("*/*.yaml"):
            cardanopy_config = CardanoPyConfig()
            cardanopy_config.load(file)

            self.assertEqual(cardanopy_config.apiVersion, "cardanopy.node.config.v1")
            self.assertIsInstance(cardanopy_config.network, str)
            self.assertEqual(cardanopy_config.configPath, "/home/ada/app/config.json")
            self.assertEqual(cardanopy_config.topologyPath, "/home/ada/app/topology.json")
            self.assertEqual(cardanopy_config.databasePath, "/home/ada/app/db")
            self.assertEqual(cardanopy_config.socketPath, "/home/ada/app/node.socket")
            self.assertEqual(cardanopy_config.hostAddr, "0.0.0.0")
            self.assertEqual(cardanopy_config.port, 3001)
            # self.assertEqual(cardanopy_config.shelleyKesKey, "/home/ada/app/bp/kes.skey")
            # self.assertEqual(cardanopy_config.shelleyVrfKey, "/home/ada/app/bp/vrf.skey")
            # self.assertEqual(cardanopy_config.shelleyOperationalCertificate, "/home/ada/app/bp/node.cert")
            self.assertIsInstance(cardanopy_config.docker.name, str)
            self.assertEqual(cardanopy_config.docker.image, f"floydcraft/cardano-py-slim:{CardanoPyCommon.get_version()}")
            self.assertEqual(cardanopy_config.docker.rootVolume, "/home/ada/app")
            self.assertEqual(cardanopy_config.docker.mount, True)
            self.assertIsInstance(cardanopy_config.k8s.namespace, str)



import click

from .run_cmd import run_cmd
from .stop_cmd import stop_cmd


@click.group("docker")
@click.pass_context
def docker_cmd(ctx):
    """Docker helper command"""
    pass

# export
docker_cmd.add_command(run_cmd)
docker_cmd.add_command(stop_cmd)



import subprocess
from pathlib import Path
from ..core.cardanopy_config import CardanoPyConfig


class DockerHelper(object):

    @staticmethod
    def is_container_running(container_name: str, dry_run=False):
        docker_cmd = ["docker",
                     "ps",
                     "-q",
                     "-f", f"name=^/{container_name}$"]
        if dry_run:
            print(" ".join(docker_cmd))

        try:
            result = subprocess.run(docker_cmd, stdout=subprocess.PIPE).stdout.decode('utf-8')
            return len(result) > 0
        except Exception as ex:
            raise ValueError(f"DockerHelper:is_container_running('{container_name}') failed. {type(ex).__name__} {ex.args}")

    @staticmethod
    def is_container_exited(container_name: str, dry_run=False):
        docker_cmd = ["docker",
                     "ps",
                     "-aq",
                     "-f", f"name=^/{container_name}$"]
        if dry_run:
            print(" ".join(docker_cmd))

        try:
            result = subprocess.run(docker_cmd, stdout=subprocess.PIPE).stdout.decode('utf-8')
            return len(result) > 0
        except Exception as ex:
            raise ValueError(f"DockerHelper:is_container_exited('{container_name}') failed. {type(ex).__name__} {ex.args}")

    @staticmethod
    def remove_container(container_name: str, dry_run=False):
        docker_cmd = ["docker",
                       "rm",
                       container_name]
        if dry_run:
            print(" ".join(docker_cmd))
        else:
            try:
                subprocess.run(docker_cmd)
            except Exception as ex:
                raise ValueError(f"DockerHelper:remove_container('{container_name}') failed. {type(ex).__name__} {ex.args}")

    @staticmethod
    def stop_container(container_name: str, dry_run=False):
        docker_cmd = ["docker",
                      "stop",
                      container_name]
        if dry_run:
            print(" ".join(docker_cmd))
        else:
            try:
                subprocess.run(docker_cmd)
            except Exception as ex:
                raise ValueError(
                    f"DockerHelper:stop_container('{container_name}') failed. {type(ex).__name__} {ex.args}")

    @staticmethod
    def pull_container(docker_image: str, dry_run=False):
        docker_cmd = ["docker",
                      "pull",
                      docker_image]
        if dry_run:
            print(" ".join(docker_cmd))
        else:
            try:
                subprocess.run(docker_cmd)
            except Exception as ex:
                raise ValueError(
                    f"DockerHelper:pull('{docker_image}') failed. {type(ex).__name__} {ex.args}")

    @staticmethod
    def exec_bash(container_name: str, target_config_dir: Path, dry_run=False):

        target_config_dir = CardanoPyConfig.try_get_valid_config_dir(target_config_dir)

        docker_cmd = ["docker",
                      "exec",
                      "-it",
                      container_name,
                      "/bin/bash"]
        if dry_run:
            print(" ".join(docker_cmd))
        else:
            try:
                subprocess.run(docker_cmd, cwd=target_config_dir)
            except Exception as ex:
                raise ValueError(f"DockerHelper:exec_bash('{container_name}') failed. {type(ex).__name__} {ex.args}")

    @staticmethod
    def run_cardano_node(container_name: str,
                         target_config_dir: Path,
                         socket_path: str,
                         network: str,
                         port: int,
                         docker_root_volume: str,
                         docker_image: str,
                         docker_mount: bool = False,
                         bash=False,
                         dry_run=False):

        target_config_dir = CardanoPyConfig.try_get_valid_config_dir(target_config_dir)

        docker_cmd = list(filter(None,
                                ["docker",
                                "run",
                                "--name", container_name,
                                "-d" if not bash else None,
                                "--env", f"CARDANO_NODE_SOCKET_PATH={socket_path}",
                                "--env", f"CARDANO_NETWORK={network}",
                                "-p", f"{port}:{port}",
                                "-v" if docker_mount else None,
                                f"{target_config_dir.absolute()}:{docker_root_volume}" if docker_mount else None,
                                "-it" if bash else None,
                                "--entrypoint",
                                "/bin/bash" if bash else "cardanopy",
                                docker_image,
                                "run" if not bash else None,
                                docker_root_volume if not bash else None]))

        if dry_run:
            print(" ".join(docker_cmd))
        else:
            try:
                subprocess.run(docker_cmd, cwd=target_config_dir)
            except Exception as ex:
                raise ValueError(f"DockerHelper:run_cardano_node('{container_name}', '{target_config_dir}') failed. {type(ex).__name__} {ex.args}")


import yaml
from pathlib import Path
import jsonschema
import json


class CardanoPyConfig(object):
    config = None
    config_resolved = None

    @staticmethod
    def try_get_valid_config_dir(target_config_dir_or_file):
        if isinstance(target_config_dir_or_file, str):
            target_config_dir_or_file = Path(target_config_dir_or_file)

        if target_config_dir_or_file.is_dir():
            target_config = target_config_dir_or_file.joinpath("cardanopy.yaml")
            target_config_dir = target_config_dir_or_file
        else:
            target_config = target_config_dir_or_file
            target_config_dir = target_config_dir_or_file.parent

        if not target_config.is_file():
            raise ValueError(f"Target config '{target_config}' is not a file. e.g., 'cardanopy.yaml'")

        if not target_config.exists():
            raise ValueError(f"Target file '{target_config}' does not exist.")

        return target_config_dir

    @staticmethod
    def try_get_valid_config_file(target_config_dir_or_file):
        if isinstance(target_config_dir_or_file, str):
            target_config_dir_or_file = Path(target_config_dir_or_file)

        if target_config_dir_or_file.is_dir():
            target_config = target_config_dir_or_file.joinpath("cardanopy.yaml")
        else:
            target_config = target_config_dir_or_file

        if not target_config.is_file():
            raise ValueError(f"Target config '{target_config}' is not a file. e.g., 'cardanopy.yaml'")

        if not target_config.exists():
            raise ValueError(f"Target file '{target_config}' does not exist.")

        return target_config

    def substitute(self, config_str, subs: tuple):
        substitutions = self.config.get('substitutions')
        if not substitutions or not config_str:
            return config_str

        for sub in subs:
            if "=" in sub:
                sub_split = sub.split("=")
                sub_key = sub_split[0] if len(sub_split) > 0 else None
                sub_value = sub_split[1] if len(sub_split) > 1 else None
                sub_key = sub_key.strip()
                sub_value = sub_value.strip()
                if sub_value and (sub_value.endswith('\'') or sub_value.endswith('\"')):
                    sub_value = sub_value[:-1]
                if sub_value and (sub_value.startswith('\'') or sub_value.startswith('\"')):
                    sub_value = sub_value[1:]

                if sub_key and not sub_key.startswith('_'):
                    raise ValueError(f"Substitutions invalid format: prefix of underscore is required.\n"
                                     f"sub_key='{sub_key}' and  sub_value='{sub_value}'\n"
                                     f"Should be: --sub _KEY_EXAMPLE='value_example'.")

                if sub_key:
                    config_str = config_str.replace(f"${sub_key}", f"{sub_value}")
                else:
                    raise ValueError(f"Substitutions invalid format: key shouldn't be None/null or Empty\n"
                                     f"sub_key='{sub_key}' and  sub_value='{sub_value}'\n"
                                     f"Should be: --sub _KEY_EXAMPLE='value_example'")
            else:
                raise ValueError(f"Substitutions invalid format: key=value should be seperated with an equal sign\n"
                                 f"sub='{sub}'\n"
                                 f"Should be: --sub _KEY_EXAMPLE='value_example'")

        for sub_key, sub_value in substitutions.items():
            # print(f"before\n{config_str}")
            config_str = config_str.replace(f"${sub_key}", f"{sub_value}")
            # print(f"after\n{config_str}")

        return config_str

    def load(self, target_config_file, subs: tuple = tuple()):
        if isinstance(target_config_file, str):
            target_config_file = Path(target_config_file)

        if not target_config_file.is_file():
            raise ValueError(f"Target config '{target_config_file}' is not a file. e.g., 'cardanopy.yaml'")

        if not target_config_file.exists():
            raise ValueError(f"Target file '{target_config_file}' does not exist.")

        try:
            with open(target_config_file, "r") as file:
                config_str = file.read()
                self.config = yaml.full_load(config_str)
                config_resolved_str = self.substitute(config_str, subs)
                self.config_resolved = yaml.full_load(config_resolved_str)
        except Exception as ex:
            raise ValueError(f"Failed to load config. '{target_config_file}'. {type(ex).__name__} {ex.args}")

        schema_file = Path(__file__).parent.joinpath("../data/schemas").absolute() / f"{self.apiVersion}.json"

        try:
            with open(schema_file, "r") as file:
                schema = json.loads(file.read())
        except Exception as ex:
            raise ValueError(f"Failed to load schema. '{target_config_file}'. {type(ex).__name__} {ex.args}")

        try:
            jsonschema.validate(instance=self.config_resolved, schema=schema)
        except Exception as ex:
            raise ValueError(f"Failed to validate config. '{target_config_file}'. {type(ex).__name__} {ex.args}")

    # def set(self, property_name: str, property_value):
    #     self.config[property_name] = property_value
    #     self.config_resolved[property_name] = copy.deepcopy(self.config[property_name])
    #
    # def get(self, property_name: str):
    #     return self.config_resolved[property_name]

    def save(self, target_config_file):
        if isinstance(target_config_file, str):
            target_config_file = Path(target_config_file)

        # if not target_config.is_file():
        #     raise ValueError(f"Target config '{target_config}' is not a file. e.g., 'cardanopy.yaml'")
        #
        # if not target_config.exists():
        #     raise ValueError(f"Target file '{target_config}' does not exist.")
        schema_file = Path(__file__).parent.joinpath("../data/schemas").absolute() / f"{self.apiVersion}.json"

        try:
            with open(schema_file, "r") as file:
                schema = json.loads(file.read())
        except Exception as ex:
            raise ValueError(f"Failed to load schema. '{target_config_file}'. {type(ex).__name__} {ex.args}")

        try:
            jsonschema.validate(instance=self.config_resolved, schema=schema)
        except Exception as ex:
            raise ValueError(f"Failed to validate config. '{target_config_file}'. {type(ex).__name__} {ex.args}")

        try:
            with open(target_config_file, "w") as file:
                yaml.dump(self.config, file)
        except Exception as ex:
            raise ValueError(f"Failed to save. '{target_config_file}'. {type(ex).__name__} {ex.args}")

    def get_api_version(self):
        return self.config_resolved.get('apiVersion')

    apiVersion = property(get_api_version)

    def get_substitutions(self):
        return self.config_resolved.get('substitutions')

    substitutions = property(get_substitutions)

    def get_network(self):
        return self.config_resolved.get('network')

    network = property(get_network)

    def get_config_path(self):
        return self.config_resolved.get('configPath')

    configPath = property(get_config_path)

    def get_topology_path(self):
        return self.config_resolved.get('topologyPath')

    topologyPath = property(get_topology_path)

    def get_database_path(self):
        return self.config_resolved.get('databasePath')

    databasePath = property(get_database_path)

    def get_host_addr(self):
        return self.config_resolved.get('hostAddr')

    hostAddr = property(get_host_addr)

    def get_port(self):
        return self.config_resolved.get('port')

    port = property(get_port)

    def get_socket_path(self):
        return self.config_resolved.get('socketPath')

    socketPath = property(get_socket_path)

    def get_shelley_kes_key(self):
        return self.config_resolved.get('shelleyKesKey')

    shelleyKesKey = property(get_shelley_kes_key)

    def get_shelley_vrf_key(self):
        return self.config_resolved.get('shelleyVrfKey')

    shelleyVrfKey = property(get_shelley_vrf_key)

    def get_shelley_operational_certificate(self):
        return self.config_resolved.get('shelleyOperationalCertificate')

    shelleyOperationalCertificate = property(get_shelley_operational_certificate)

    def get_docker(self):
        return DockerConfig(self.config_resolved.get('docker'))

    docker = property(get_docker)

    def get_k8s(self):
        return KubernetesConfig(self.config_resolved.get('k8s'))

    k8s = property(get_k8s)


class DockerConfig(object):
    config = None

    def __init__(self, config):
        self.config = config

    def get_name(self):
        return self.config.get('name')

    name = property(get_name)

    def get_image(self):
        return self.config.get('image')

    image = property(get_image)

    def get_root_volume(self):
        return self.config.get('rootVolume')

    rootVolume = property(get_root_volume)

    def get_mount(self):
        return self.config.get('mount', False)

    mount = property(get_mount)


class KubernetesConfig(object):
    config = None

    def __init__(self, config):
        self.config = config

    def get_namespace(self):
        return self.config.get('namespace')

    namespace = property(get_namespace)


import unittest
from click.testing import CliRunner
from ...create_cmd import create_cmd
from core.substitution import Substitution
from core.cardanopy_config import CardanoPyConfig
from pathlib import Path
import tempfile
import shutil
import os


class TestSubstitution(unittest.TestCase):

    def setUp(self):
        self.test_dir = Path(tempfile.mkdtemp())
        os.chdir(self.test_dir)
        # print(f"test_dir='{self.test_dir}'")

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    def test_substitution(self):
        app_dir = self.test_dir.joinpath('test-app')

        runner = CliRunner()
        create_result = runner.invoke(create_cmd, ['--template',
                                                    'basic',
                                                    '--network',
                                                    'testnet',
                                                    str(app_dir)])
        assert create_result.exit_code == 0

        target_config_dir = CardanoPyConfig.try_get_valid_config_dir(app_dir)
        target_config_file = CardanoPyConfig.try_get_valid_config_file(app_dir)

        cardanopy_config = CardanoPyConfig()
        cardanopy_config.load(target_config_file, ("_NAME=test_app", "_TAG=test_tag"))

        Substitution.generate(False, target_config_dir, cardanopy_config)



import subprocess
from typing import NamedTuple
from .cardanopy_error import CardanoPyError


class CliResult(NamedTuple):
    stdout: bytes
    stderr: bytes
    encoding: str = "utf-8"

    def get_stdout_str(self):
        return self.stdout.decode(self.encoding) if self.stdout else None

    stdout_str = property(get_stdout_str)

    def get_stderr_str(self):
        return self.stderr.decode(self.encoding) if self.stdout else None

    stderr_str = property(get_stderr_str)


class CliBase(NamedTuple):

    @staticmethod
    def execute(cmd: list, dry_run: bool = False, encoding: str = "utf-8"):
        try:
            if dry_run:
                print(" ".join(cmd))
                stdout, stderr = b""
            else:
                p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                stdout, stderr = p.communicate()

                if p.returncode != 0:
                    raise CardanoPyError(f" {stderr.decode(encoding)}", p.returncode)

        except subprocess.CalledProcessError as cpe:
            raise CardanoPyError(f"{' '.join(cmd)} failed: {cpe.returncode} {cpe.output}", cpe.returncode)
        except Exception as ex:
            raise CardanoPyError(f"{' '.join(cmd)} failed: {type(ex).__name__} {ex.args}", 1)

        return CliResult(stdout=stdout, stderr=stderr, encoding=encoding)




import click
import subprocess


@click.group("stake-address")
@click.pass_context
def stake_address_cmd(ctx):
    """Stake address commands"""


@stake_address_cmd.command("key-gen")
@click.option('--verification-key-file', 'verification_key_file', required=True, type=str, help='Output filepath of the verification key.')
@click.option('--signing-key-file', 'signing_key_file', required=True, type=str, help='Output filepath of the signing key.')
@click.pass_context
def key_gen_cmd(ctx, verification_key_file, signing_key_file):
    """Create a stake address key pair"""

    try:
        subprocess.run(["cardano-cli",
                        "stake-address",
                        "key-gen",
                        "--verification-key-file", verification_key_file,
                        "--signing-key-file", signing_key_file
                        ])
    except Exception as ex:
        ctx.fail(f"cli:stake_address_cmd:key_gen_cmd(verification_key_file={verification_key_file}, "
                 f"signing_key_file={signing_key_file}) failed: {type(ex).__name__} {ex.args}")
        return 1



from ..core.cli_base import CliBase
from .cardano_node import CardanoNode


class CardanoCli(object):

    @staticmethod
    def execute(cmd: list, dry_run: bool = False,
                include_network: bool = False, include_era: bool = False, encoding: str = "utf-8"):
        if not dry_run:
            CardanoNode.validate_environment()

        if include_network:
            cmd = cmd + CardanoCli.get_network_cli_args()

        if include_era:
            cmd = cmd + CardanoCli.get_era_cli_args()

        return CliBase.execute(cmd=cmd, dry_run=dry_run, encoding=encoding)

    @staticmethod
    def get_network_cli_args():
        # TODO:bfloyd change to use yaml config instead?
        # TODO:bfloyd allow for custom testnet magic?
        return ["--mainnet"] if CardanoNode.get_network() == "mainnet" else ["--testnet-magic", "1097911063"]

    @staticmethod
    def get_era_cli_args():
        # TODO:bfloyd implement
        return []







import click
import subprocess
from pathlib import Path
from ..core.cardanopy_config import CardanoPyConfig
from .docker_helper import DockerHelper


@click.command("stop")
@click.option('-d', '--dry-run', 'dry_run', is_flag=True, help="print the mutable commands")
@click.option('-s', '--sub', 'subs', multiple=True, type=str, default=tuple(), help="Substitutions for configs")
@click.argument('target_config_dir_or_file', type=str)
@click.pass_context
def stop_cmd(ctx, dry_run, subs, target_config_dir_or_file):
    """Docker Stop helper command"""

    try:
        target_config_file = CardanoPyConfig.try_get_valid_config_file(target_config_dir_or_file)

        cardanopy_config = CardanoPyConfig()
        cardanopy_config.load(target_config_file, subs)

        if DockerHelper.is_container_running(cardanopy_config.docker.name, dry_run):
            DockerHelper.stop_container(cardanopy_config.docker.name, dry_run)

        if DockerHelper.is_container_exited(cardanopy_config.docker.name, dry_run):
            DockerHelper.remove_container(cardanopy_config.docker.name, dry_run)
    except Exception as ex:
        ctx.fail(f"docker_cmd:stop_cmd(dry_run={dry_run}, target_config_dir_or_file='{target_config_dir_or_file}') failed: {type(ex).__name__} {ex.args}")
        return 1





class CardanoPyError(Exception):
    message: None
    return_code: 0

    def __init__(self, message: str, return_code: int = 0):
        self.message = message
        self.return_code = return_code



import click
from pathlib import Path
import shutil


def get_template_dir(network: str, template: str):
    return Path(__file__).parent.joinpath("data/templates").joinpath(network).joinpath(template)


@click.command("create")
@click.option('-t', '--template', 'template', required=True,
              type=click.Choice(['basic', 'bp-k8s', 'relay-k8s'], case_sensitive=False),
              help='template type to create.')
@click.option('-n', '--network', 'network', default="testnet",
              type=click.Choice(['testnet', 'mainnet'], case_sensitive=False), help='network type to create.')
@click.option('-d', '--dry-run', 'dry_run', is_flag=True, help="print the mutable commands")
@click.option('-q', '--quite', 'quite', is_flag=True, help="do not report warnings")
@click.argument('target_dir', type=str)
@click.pass_context
def create_cmd(ctx, template, network, dry_run, quite, target_dir):
    """Create command"""

    try:
        target_dir = Path(target_dir)
        network = network.lower()
        template = template.lower()
        template_dir = get_template_dir(network, template)

        if target_dir.exists():
            if not quite:
                ctx.fail(f"Failed to create. Directory '{target_dir}' already exists.")
                return 1
            else:
                return 0

        if not template_dir.is_dir() or not template_dir.exists():
            ctx.fail(f"Failed to create. Unable to locate template '{template}' for network '{network}'.")
            return 1

        if dry_run:
            print(f"copy 'template' directory from '{template_dir}' to '{target_dir}'")
        else:
            shutil.copytree(template_dir, target_dir)

        if not dry_run:
            print(f"Created cardano defaults from '{template}' template for network '{network}': '{target_dir}'")
    except Exception as ex:
        ctx.fail(f"create_cmd(template={template}, network={network}, dry_run={dry_run}, target_dir='{target_dir}') failed: {type(ex).__name__} {ex.args}")
        return 1



#!/usr/bin/env python

from setuptools import setup, find_packages
from pathlib import Path

datadir = Path(__file__).parent / 'cardanopy' / 'data'
files = ['data/' + str(p.relative_to(datadir)) for p in datadir.rglob('*')]

setup(name='cardanopy',
      version='0.1.7',
      description='Cardano CLI tools for python3',
      author='Bourke Floyd',
      author_email='chbfiv@floydcraft.com',
      url='https://github.com/floydcraft/cardano-py',
      # package_dir={'cardanopy': '.'},
      package_data={'cardanopy': files},
      packages=find_packages(exclude=['tests']),
      keywords='cardano,ada,cli',
      python_requires='>=3.5.3,<4',
      install_requires=[
        'click~=7.1.2',
        'pyyaml~=5.4.1',
        'jsonschema~=3.2.0'
      ],
      entry_points={
          'console_scripts': [
              'cardanopy=cardanopy:main',
          ],
      },
      project_urls={
          'Bug Reports': 'https://github.com/floydcraft/cardano-py/issues',
          'Chat': 'https://discord.gg/FyDz4Xrt4x',
          'Source': 'https://github.com/floydcraft/cardano-py',
      }
)



import click
import time
from .node.query import query_tip
from .core.cardanopy_config import CardanoPyConfig


@click.command("healthcheck")
@click.option('-s', '--sub', 'subs', multiple=True, type=str, default=tuple(), help="Substitutions for configs")
@click.option('-d', '--dry-run', 'dry_run', is_flag=True, help="print the mutable commands")
@click.option('-t', '--timeout', 'timeout', default=60, type=int, help="timeout for healthcheck second check")
@click.argument('target_dir', type=str)
@click.pass_context
def healthcheck_cmd(ctx, dry_run, subs, timeout, target_dir):
    """Check if the cardano network is healthy"""

    try:
        target_config_file = CardanoPyConfig.try_get_valid_config_file(target_dir)

        cardanopy_config = CardanoPyConfig()
        cardanopy_config.load(target_config_file, subs)

        tip = query_tip(dry_run)
        if len(tip) > 0 and "blockNo" in tip:
            return 0

        time.sleep(timeout)

        tip = query_tip(dry_run)
        if len(tip) > 0 and "blockNo" in tip:
            return 0

    except Exception as ex:
        ctx.fail(f"healthcheck_cmd(dry_run={dry_run}, timeout={timeout}, target_dir='{target_dir}') failed: {type(ex).__name__} {ex.args}")
        return 1

    print("unable to query tip for healthcheck")
    return 1
